#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2012-2016 Matt Martz
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import warnings

DEPRECATED_MSG = ('The file speedtest_cli.py has been deprecated in favor of '
                  'speedtest.py\nand is available for download at:\n\n'
                  'https://raw.githubusercontent.com/sivel/speedtest-cli/'
                  'master/speedtest.py')


if __name__ == '__main__':
    raise SystemExit(DEPRECATED_MSG)
else:
    try:
        from speedtest import *
    except ImportError:
        raise SystemExit(DEPRECATED_MSG)
    else:
        warnings.warn(DEPRECATED_MSG, UserWarning)
